ALREADY_INITIALIZED_MSG: str = "The current directory is not initialized. Please run [bold]\"compiloor init\"[/bold] to initialize the current directory."
NOT_INITIALIZED_MSG: str = "The current directory is already initialized. If you want to override the current directory, run [bold]\"compiloor init --force\"[/bold]."
