TABLE_HTML_OUTPUT_FORMAT_SETTING = "html"

PAGE_NUMBER_FONT_SIZE = 13

PRIMARY_COLOR_IN_PERCENTAGES = (0.129, 0.145, 0.161)

REPORT_EXTENSION = ".pdf"

MAIN_REPORT_SECTIONS = [
    "findings",
    "total_findings_amount",
    "stylesheet",
]